package com.hncy.sjdz.constant.enums;

import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

import java.util.Objects;

/**
 * GirdTypeEnum 网格类型
 * <p>
 * Enum("name","value")
 * <p>
 * ----------------------------------
 * <p>
 * PROVINCE("省界", "1")
 * <p>
 * CITY("市界", "2")
 * <p>
 * DISTRICTS_CITIES_COUNTIES("区市县界", "3")
 * <p>
 * SUBDISTRICT_OFFICE("街道办界", "4")
 * <p>
 * COMMUNITY("社区界", "5")
 * <p>
 * CUSTOM("自定义界限", "6")
 *
 * @Author: lm
 * @Date: 2023/12/05
 */
@Getter
@RequiredArgsConstructor
public enum GirdTypeEnum {

    /**
     * 省界 v:"1"
     */
    PROVINCE("省界", "1"),
    /**
     * 市界 v:"2"
     */
    CITY("市界", "2"),
    /**
     * 区市县界 v:"3"
     */
    DISTRICTS_CITIES_COUNTIES("区市县界", "3"),
    /**
     * 街道办界 v:"4"
     */
    SUBDISTRICT_OFFICE("街道办界", "4"),
    /**
     * 社区界 v:"5"
     */
    COMMUNITY("社区界", "5"),
    /**
     * 道路 v:"6"
     */
    ROAD("道路", "6"),
    /**
     * 重要区域 v:"19"
     */
    IMPORTANT_AREA("重要区域", "19"),
    /**
     * 自定义界限 v:"66"
     */
    CUSTOM("自定义界限", "66"),;

    private final String name;

    @JsonValue
    private final String value;

    public static String getName(String value) {
        GirdTypeEnum[] enums = GirdTypeEnum.values();
        for (GirdTypeEnum nowEnum : enums) {
            if (nowEnum.getValue().equals(value)) {
                return nowEnum.getName();
            }
        }
        return "/";
    }

    public static String convertName(GirdTypeEnum enumObj) {
        return Objects.isNull(enumObj) ? "/" : enumObj.getName();
    }

}