package com.hncy.sjdz.constant.enums;

import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

import java.util.Objects;

/**
 * PhysicalSignIndexOptionTypeEnum 体征指数选项类型
 * <p>
 * Enum("name","value")
 * <p>
 * ----------------------------------
 * <p>
 * INSIDE("全部", "A")
 * <p>
 * INSIDE("指数", "0")
 * <p>
 * QUESTION("问题", "Q")
 * <p>
 * OBJECTIVE("对象", "O")
 *
 * @Author: lm
 * @Date: 2024/3/13
 */
@Getter
@RequiredArgsConstructor
public enum PhysicalSignIndexOptionTypeEnum {

    /**
     * 全部 v:"A"
     */
    ALL("全部", "A"),
    /**
     * 指数 v:"0"
     */
    INDEX("指数", "0"),
    /**
     * 问题 v:"1"
     */
    QUESTION("问题", "Q"),
    /**
     * 对象 v:"1"
     */
    OBJECTIVE("对象", "O");

    private final String name;

    @JsonValue
    private final String value;

    public static String getName(String value) {
        PhysicalSignIndexOptionTypeEnum[] enums = PhysicalSignIndexOptionTypeEnum.values();
        for (PhysicalSignIndexOptionTypeEnum nowEnum : enums) {
            if (nowEnum.getValue().equals(value)) {
                return nowEnum.getName();
            }
        }
        return "/";
    }

    public static String convertName(PhysicalSignIndexOptionTypeEnum enumObj) {
        return Objects.isNull(enumObj) ? "/" : enumObj.getName();
    }

}