package com.hncy.sjdz.constant.enums;

import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

import java.util.Objects;

/**
 * 体征指数 体征对象级别
 * <p>
 * Enum("name","value")
 * <p>
 * ----------------------------------
 * <p>
 * DISTRICTS_CITIES_COUNTIES("区市县", "1")
 * <p>
 * SUBDISTRICT_OFFICE("街道办", "2")
 * <p>
 * COMMUNITY("社区", "3")
 * <p>
 * ROAD("道路", "4")
 *
 * @Author: lm
 * @Date: 2024/3/27
 */
@Getter
@RequiredArgsConstructor
public enum PhysicalSignObjectTypeEnum {

    /**
     * 区市县 v:"1"
     */
    DISTRICTS_CITIES_COUNTIES("区市县", "1"),
    /**
     * 街道办 v:"2"
     */
    SUBDISTRICT_OFFICE("街道办", "2"),
    /**
     * 社区 v:"3"
     */
    COMMUNITY("社区", "3"),
    /**
     * 道路 v:"4"
     */
    ROAD("道路", "4"),
    /**
     * 重要区域 v:"19"
     */
    IMPORTANT_AREA("重要区域", "19"),;

    private final String name;

    @JsonValue
    private final String value;

    public static String getName(String value) {
        PhysicalSignObjectTypeEnum[] enums = PhysicalSignObjectTypeEnum.values();
        for (PhysicalSignObjectTypeEnum nowEnum : enums) {
            if (nowEnum.getValue().equals(value)) {
                return nowEnum.getName();
            }
        }
        return "/";
    }

    public static String convertName(PhysicalSignObjectTypeEnum enumObj) {
        return Objects.isNull(enumObj) ? "/" : enumObj.getName();
    }

}