package com.hncy.sjdz.constant.enums;

import com.fasterxml.jackson.annotation.JsonValue;
import lombok.Getter;
import lombok.RequiredArgsConstructor;

import java.util.Objects;

/**
 * ResponsibilitySystemModeEnum 责任方式
 * <p>
 * Enum("name","value")
 * <p>
 * ----------------------------------
 * <p>
 * PERSON("人工巡查", "1")
 * <p>
 * VEHICLE_MOUNTED("车载巡查", "2")
 *
 * @Author: lm
 * @Date: 2023/11/30
 */
@Getter
@RequiredArgsConstructor
public enum ResponsibilitySystemModeEnum {

    /**
     * 人工巡查 v:"1"
     */
    PERSON("人工巡查", "1"),
    /**
     * 车载巡查 v:"2"
     */
    VEHICLE_MOUNTED("车载巡查", "2"),
    /**
     * 智能感知 v:"3"
     */
    INTELLISENSE("智能感知", "3"),
    /**
     * 固定摄像头 v:"4"
     */
    FIXED_CAMERA("固定摄像头", "4");

    private final String name;

    @JsonValue
    private final String value;

    public static String getName(String value) {
        ResponsibilitySystemModeEnum[] enums = ResponsibilitySystemModeEnum.values();
        for (ResponsibilitySystemModeEnum nowEnum : enums) {
            if (nowEnum.getValue().equals(value)) {
                return nowEnum.getName();
            }
        }
        return "/";
    }

    public static String convertName(ResponsibilitySystemModeEnum enumObj) {
        return Objects.isNull(enumObj) ? "/" : enumObj.getName();
    }

}