
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.Assert;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.math.BigDecimal;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Author tsj
 * @Date 2024/1/17/017 10:40
 * @Version 1.0
 */
@Slf4j
public class commonTest {

    private final static int TWO_SIDE_VALID = 0;

    private final static int LEFT_VALID = 1;

    private final static int RIGHT_VALID = 2;

    private final static int NONE_VALID = 3;

    private final static String FORMULA_ERROR_NO_VALID = "不包含可用的公式，请修改";

    private final static String FORMULA_ERROR_WITH_BLIND_POINTS = "公式存在以下盲点，请确认是否修改：";

    private final static String FORMULA_NO_ERROR = "公式正确";

    public static void main(String[] args) {
        String formula = "";
        double x = 1.5;

        // 测试用例1 - 单边：操作数为空
        formula = "{2<=x<,8}{-2<=x<0,3}{0<=x<2,5}";
        log.info("+++ formula 1:{}", formula);
        assertAndLogPiecewiseFunction(x, formula, 5);
        assertAndLogPiecewiseFunction(-1.5, formula, 3);
        assertAndLogPiecewiseFunction(0, formula, 5);
        assertAndLogPiecewiseFunction(1.5, formula, 5);
        assertAndLogPiecewiseFunction(2, formula, 8);
        assertAndLogPiecewiseFunction(99, formula, 8);
        assertAndLogPiecewiseFunction(100, formula, 8);
        assertAndLogPiecewiseFunction(200, formula, 8);
        assertAndLogPiecewiseFunction(-200, formula, Double.NaN);
        // 结构化分段式公式：
        testStructuredPiecewiseFunction(formula);
        // 自动校验分段式公式：逻辑检查和边界检查
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例2 -- 单边：操作数为空，不等号也是空
        formula = "{-2<=x<0,3}{0<=x<2,5}{2<=x,8}";
        log.info("+++ formula 2:{}", formula);
        assertAndLogPiecewiseFunction(x, formula, 5);
        assertAndLogPiecewiseFunction(-1.5, formula, 3);
        assertAndLogPiecewiseFunction(0, formula, 5);
        assertAndLogPiecewiseFunction(1.5, formula, 5);
        assertAndLogPiecewiseFunction(2, formula, 8);
        assertAndLogPiecewiseFunction(99, formula, 8);
        assertAndLogPiecewiseFunction(100, formula, 8);
        assertAndLogPiecewiseFunction(200, formula, 8);
        assertAndLogPiecewiseFunction(-200, formula, Double.NaN);
        // 结构化分段式公式：
        testStructuredPiecewiseFunction(formula);
        // 自动校验分段式公式：逻辑检查和边界检查
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例3 -- 只双边不等式
        formula = "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}";
        log.info("+++ formula 3:{}", formula);
        assertAndLogPiecewiseFunction(x, formula, 5);
        assertAndLogPiecewiseFunction(-1.5, formula, 3);
        assertAndLogPiecewiseFunction(0, formula, 5);
        assertAndLogPiecewiseFunction(1.5, formula, 5);
        assertAndLogPiecewiseFunction(2, formula, 8);
        assertAndLogPiecewiseFunction(99, formula, 8);
        assertAndLogPiecewiseFunction(100, formula, Double.NaN);
        assertAndLogPiecewiseFunction(200, formula, Double.NaN);
        assertAndLogPiecewiseFunction(-200, formula, Double.NaN);
        // 结构化分段式公式：
        testStructuredPiecewiseFunction(formula);
        // 自动校验分段式公式：逻辑检查和边界检查
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例4 -- 不完整的公式：
        formula = "{-2<=x<0,3}{0<=x<2,5}{<=x,8}";
        log.info("+++ formula 4:{}", formula);
        assertAndLogPiecewiseFunction(x, formula, 5);
        assertAndLogPiecewiseFunction(-1.5, formula, 3);
        assertAndLogPiecewiseFunction(0, formula, 5);
        assertAndLogPiecewiseFunction(1.5, formula, 5);
        assertAndLogPiecewiseFunction(2, formula, Double.NaN);
        assertAndLogPiecewiseFunction(99, formula, Double.NaN);
        assertAndLogPiecewiseFunction(100, formula, Double.NaN);
        assertAndLogPiecewiseFunction(200, formula, Double.NaN);
        assertAndLogPiecewiseFunction(-200, formula, Double.NaN);
        // 结构化分段式公式：
        testStructuredPiecewiseFunction(formula);
        // 自动校验分段式公式：逻辑检查和边界检查
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例5 -- 不完整的公式：
        formula = "{-2<=x<0,3}{0<=x<2,5}{34x,8}";
        log.info("+++ formula 5:{}", formula);
        assertAndLogPiecewiseFunction(x, formula, 5);
        assertAndLogPiecewiseFunction(-1.5, formula, 3);
        assertAndLogPiecewiseFunction(0, formula, 5);
        assertAndLogPiecewiseFunction(1.5, formula, 5);
        assertAndLogPiecewiseFunction(2, formula, Double.NaN);
        assertAndLogPiecewiseFunction(99, formula, Double.NaN);
        assertAndLogPiecewiseFunction(100, formula, Double.NaN);
        assertAndLogPiecewiseFunction(200, formula, Double.NaN);
        assertAndLogPiecewiseFunction(-200, formula, Double.NaN);
        // 结构化分段式公式：
        testStructuredPiecewiseFunction(formula);
        // 自动校验分段式公式：逻辑检查和边界检查
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例6 -- 不完整的公式：
        formula = "{-2<=x<0,3}{0<=x<2,5}{x,8}";
        log.info("+++ formula 6:{}", formula);
        assertAndLogPiecewiseFunction(x, formula, 5);
        assertAndLogPiecewiseFunction(-1.5, formula, 3);
        assertAndLogPiecewiseFunction(0, formula, 5);
        assertAndLogPiecewiseFunction(1.5, formula, 5);
        assertAndLogPiecewiseFunction(2, formula, Double.NaN);
        assertAndLogPiecewiseFunction(99, formula, Double.NaN);
        assertAndLogPiecewiseFunction(100, formula, Double.NaN);
        assertAndLogPiecewiseFunction(200, formula, Double.NaN);
        assertAndLogPiecewiseFunction(-200, formula, Double.NaN);
        // 结构化分段式公式：
        testStructuredPiecewiseFunction(formula);
        // 自动校验分段式公式：逻辑检查和边界检查
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例7 -- 自动校验功能：盲点
        formula = "{-2<=x<0,3}{1<=x<2,5}{1<=x<2,8}";
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例8 -- 自动校验功能：反向不等号1
        formula = "{-2<=x<0,3}{1<=x>2,5}";
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例9 -- 自动校验功能：反向不等号2
        formula = "{-2<=x<0,3}{1>x<2,5}";
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例9 -- 自动校验功能：不完整的公式
        formula = "{-2<=x<0,3}{0<=x<2,5}{34x,8}";
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例10 -- 自动校验功能：通过
        formula = "{-2>x,2}{-2<=x<0,3}{0<=x<2,5}{2<=x,8}";
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例11 -- 自动校验功能：全部无效
        formula = "{>x,2}{<=x,3}{x,5}{<=x<=,8}";
        log.info("=== 自动校验 公式:{}；结果：{}", formula, autoCheckCorrectnessOfFormula(formula));

        // 测试用例12 -- 测试 evaluatePiecewiseFunction 方法 --- 自动生成
        testEvaluatePiecewiseFunction();
    }

    private static void testEvaluatePiecewiseFunction() {
        // 测试常规的分段公式
        double result = evaluatePiecewiseFunction(0, "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}");
        assertEquals(5, result, 0.0001);

        result = evaluatePiecewiseFunction(-1, "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}");
        assertEquals(3, result, 0.0001);

        result = evaluatePiecewiseFunction(5, "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}");
        assertEquals(8, result, 0.0001);

        result = evaluatePiecewiseFunction(10, "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}");
        assertEquals(8, result, 0.0001);

        // 测试单边不等式
        result = evaluatePiecewiseFunction(0, "{x<0,3}{0<=x<2,5}{x>=2,8}");
        assertEquals(5, result, 0.0001);

        result = evaluatePiecewiseFunction(-1, "{x<0,3}{0<=x<2,5}{x>=2,8}");
        assertEquals(3, result, 0.0001);

        result = evaluatePiecewiseFunction(5, "{x<0,3}{0<=x<2,5}{x>=2,8}");
        assertEquals(8, result, 0.0001);

        // 测试特殊情况，例如输入值在所有条件之外
        result = evaluatePiecewiseFunction(-10, "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}");
        assertEquals(Double.NaN, result, 0.0001);

        result = evaluatePiecewiseFunction(101, "{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}");
        assertEquals(Double.NaN, result, 0.0001);

        // 测试无效的公式格式
        result = evaluatePiecewiseFunction(0, "InvalidFormula");
        assertTrue(Double.isNaN(result));

        // 测试不包含逗号和结果值的情况
        result = evaluatePiecewiseFunction(0, "{-2<=x<0}{0<=x<2}{2<=x<100}");
        assertTrue(Double.isNaN(result));
    }

    // 前端实现--语法检查： 检查分段公式的语法是否正确。确保每个条件都有相应的结果，并且条件之间使用正确的逻辑符号连接。
    // 后端支持不完整公式（理论上可以留空不填，后端有校验方法返回盲点），所以前端需要对单个公式--1、双边不等号不能反向(1<=x>2),(1>x<2)...
    // 后端进行逻辑检查和边界检查，可以自动检测整套公式存在的盲点，并返回提示给用户。
    public static String autoCheckCorrectnessOfFormula(String formula) {
        List<PiecewiseFormula> retStruct = getStructuredPiecewiseFunction(formula);
        if (null == retStruct || retStruct.size() == 0) {
            return FORMULA_ERROR_NO_VALID;
        }
        // log.info("=== autoCheckCorrectnessOfFormula:{}", formula);
        // 获取所有分界点，去重，
        double[] breakPoints = getAllBreakPoints(retStruct);
        if (breakPoints.length == 0) {
            return FORMULA_ERROR_NO_VALID;
        }
        // 使用Set进行去重
        breakPoints = deduplicateArray(breakPoints);
        // 从小到大排序
        Arrays.sort(breakPoints);

        // 生成测试点：非端头的分界点，相邻点之间增加测试点（取中间点）
        // 端头点：对最小点增加测试点（比最小点减1），对最大点增加测试点（比最大点加1）
        List<Double> testPoints = new ArrayList<>();
        // 插入测试点: 对最小点减1
        testPoints.add(breakPoints[0] - 1);
        for (int i = 0; i < breakPoints.length - 1; i++) {
            double current = breakPoints[i];
            double next = breakPoints[i + 1];
            double midPoint = calculateMidPoint(current, next);
            testPoints.add(current);
            testPoints.add(midPoint);
        }
        testPoints.add(breakPoints[breakPoints.length - 1]);
        // 插入测试点: 对最大点加1
        testPoints.add(breakPoints[breakPoints.length - 1] + 1);

        // 所有测试点去计算这个公式，判断是否有错误。
        String blindPointsStr = "";
        for (Double item : testPoints) {
            double result = evaluatePiecewiseFunction(item, formula);
            // log.info("autoCheckCorrectnessOfFormula -- x:{}, Result: {}", item,
            // result);
            if (Double.isNaN(result)) {
                blindPointsStr += item.toString() + ",";
            }
        }
        if (blindPointsStr.length() > 0) {
            return FORMULA_ERROR_WITH_BLIND_POINTS + blindPointsStr;
        }
        return FORMULA_NO_ERROR;
    }

    private static double calculateMidPoint(double a, double b) {
        BigDecimal decimalA = BigDecimal.valueOf(a);
        BigDecimal decimalB = BigDecimal.valueOf(b);
        BigDecimal midPoint = decimalA.add(decimalB).divide(BigDecimal.valueOf(2));
        return midPoint.doubleValue();
    }

    // 获取所有分界点
    private static double[] getAllBreakPoints(List<PiecewiseFormula> retStruct) {
        double[] retPoints = new double[retStruct.size() * 2];
        int index = 0;

        for (PiecewiseFormula item : retStruct) {
            if (!Double.isNaN(getBreakPoint(item.operatorLeft, item.inequalityLeft))) {
                retPoints[index++] = item.operatorLeft;
            }
            if (!Double.isNaN(getBreakPoint(item.operatorRight, item.inequalityRight))) {
                retPoints[index++] = item.operatorRight;
            }
        }
        double[] finalPoints = new double[index];

        for (int i = 0; i < index; i++) {
            finalPoints[i] = retPoints[i];
        }
        return finalPoints;
    }

    // double数组去重
    private static double[] deduplicateArray(double[] input) {
        // 使用Set进行去重
        Set<Double> uniqueSet = new HashSet<>();
        for (double element : input) {
            uniqueSet.add(element);
        }

        // 将Set转回数组
        double[] uniqueArray = new double[uniqueSet.size()];
        int index = 0;
        for (Double element : uniqueSet) {
            uniqueArray[index++] = element;
        }
        return uniqueArray;
    }

    private static double getBreakPoint(double operator, String inequality) {
        if (!Double.isNaN(operator) && null != inequality) {
            return operator;
        }
        return Double.NaN;
    }

    // 测试结构化分段式公式：
    private static void testStructuredPiecewiseFunction(String formula) {
        List<PiecewiseFormula> retStruct = getStructuredPiecewiseFunction(formula);
        for (PiecewiseFormula item : retStruct) {
            log.info("structured formula: {} {} {} {},result:{}", item.operatorLeft, item.inequalityLeft,
                    item.inequalityRight, item.operatorRight, item.result);
        }
    }

    private static void assertAndLogPiecewiseFunction(double x, String formula, double expectResult) {
        double result = evaluatePiecewiseFunction(x, formula);
        log.info("x:{}, Result: {}", x, result);
        Assert.state((expectResult == result) || (Double.isNaN(expectResult) && Double.isNaN(result)),
                "===> wrong: x=" + x + "; result=" + result + "; expectResult=" + expectResult);
    }

    /*
     * 根据分段公式和变量x的值，计算最终的值； x: 输入的x的数值 formula：
     * 分段公式，如"{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}"，包含一个或多个带x的不等式，每个不等式用大括号分隔，
     * 里面逗号后面表示满足条件时的返回值
     */
    public static double evaluatePiecewiseFunction(double x, String formula) {
        // 使用正则表达式提取每个公式
        Pattern pattern = Pattern.compile("\\{([^}]*)}");
        Matcher matcher = pattern.matcher(formula);

        while (matcher.find()) {
            String formulaPart = matcher.group(1);

            // 分割公式的条件和结果
            String[] parts = formulaPart.split(",");
            if (parts.length == 2) {
                String condition = parts[0];
                String resultStr = parts[1];

                // 使用正则表达式提取操作数和操作符 "-2<=x<0"
                Pattern patternFormula = Pattern
                    .compile("([-+]?\\d*\\.?\\d+)?(<=|>=|<|>)?x(<=|>=|<|>)?([-+]?\\d*\\.?\\d+)?");
                Matcher matcherFormula = patternFormula.matcher(condition);

                if (matcherFormula.matches()) {
                    // 先判断不等式是否双边，是左单边、右单边，然后计算
                    switch (getInequalityType(matcherFormula)) {
                        case TWO_SIDE_VALID: {
                            double a = Double.parseDouble(matcherFormula.group(1));
                            double b = Double.parseDouble(matcherFormula.group(4));
                            String inequality1 = matcherFormula.group(2);
                            String inequality2 = matcherFormula.group(3);

                            // 检查条件是否匹配
                            if (checkCondition(x, a, b, inequality1, inequality2)) {
                                return Double.parseDouble(resultStr);
                            }
                        }
                            break;
                        case LEFT_VALID: {
                            double a = Double.parseDouble(matcherFormula.group(1));
                            String inequality1 = matcherFormula.group(2);

                            // 检查条件是否匹配
                            if (checkSingleCondition(x, a, inequality1)) {
                                return Double.parseDouble(resultStr);
                            }
                        }
                            break;
                        case RIGHT_VALID: {
                            double b = Double.parseDouble(matcherFormula.group(4));
                            String inequality2 = matcherFormula.group(3);

                            // 检查条件是否匹配
                            if (checkSingleCondition(b, x, inequality2)) {
                                return Double.parseDouble(resultStr);
                            }
                        }
                            break;
                        case NONE_VALID: {

                        }
                            break;
                        default: {
                            // 出错。
                            log.error("不等式:{} 类型解析出错！", condition);
                            return Double.NaN;
                        }
                    }
                }
                else {
                    continue;
                }
            }
        }

        // 默认返回值，可以根据实际情况修改
        // 此时x不满足上述formula，也就是说formula条件不充分。
        return Double.NaN;
    }

    // 判断不等式是否双边，是左单边、右单边
    private static int getInequalityType(Matcher input) {
        boolean first = StringUtils.isNoneEmpty(input.group(1));
        boolean second = StringUtils.isNoneEmpty(input.group(2));
        boolean third = StringUtils.isNoneEmpty(input.group(3));
        boolean fourth = StringUtils.isNoneEmpty(input.group(4));
        if (first && second && third && fourth) {
            return TWO_SIDE_VALID;
        }
        if (first && second) {
            return LEFT_VALID;
        }
        if (third && fourth) {
            return RIGHT_VALID;
        }
        return NONE_VALID;
    }

    /*
     * 公式："-2<=x<0" return: true: x满足条件； false： x不满足条件
     */
    private static boolean checkCondition(double x, double a, double b, String inequality1, String inequality2) {
        boolean bRet = checkSingleCondition(x, a, inequality1);
        if (bRet) {
            return checkSingleCondition(b, x, inequality2);
        }
        return false;
    }

    /*
     * 公式："-2<=x" return: true: x满足条件； false： x不满足条件
     */
    private static boolean checkSingleCondition(double x, double a, String inequality) {
        switch (inequality) {
            case ">":
                if (a <= x) {
                    return false;
                }
                break;
            case ">=":
                if (a < x) {
                    return false;
                }
                break;
            case "<":
                if (a >= x) {
                    return false;
                }
                break;
            case "<=":
                if (a > x) {
                    return false;
                }
                break;
        }
        return true;
    }

    /*
     * 根据字符串分段公式，返回结构化后的公式； formula：
     * 分段公式，如"{-2<=x<0,3}{0<=x<2,5}{2<=x<100,8}"，包含一个或多个带x的不等式，每个不等式用大括号分隔，
     * 里面逗号后面表示满足条件时的返回值 retStruct: 用来返回结构化后的公式，列表，每个记录包含左操作数、右操作数、左操作符、右操作符 见类
     * PiecewiseFormula
     */
    public static List<PiecewiseFormula> getStructuredPiecewiseFunction(String formula) {
        List<PiecewiseFormula> retStruct = new ArrayList<>();
        // 使用正则表达式提取每个公式
        Pattern pattern = Pattern.compile("\\{([^}]*)}");
        Matcher matcher = pattern.matcher(formula);

        while (matcher.find()) {
            String formulaPart = matcher.group(1);

            // 分割公式的条件和结果
            String[] parts = formulaPart.split(",");
            if (parts.length == 2) {
                String condition = parts[0];
                String resultStr = parts[1];

                // 使用正则表达式提取操作数和操作符 "-2<=x<0"
                Pattern patternFormula = Pattern
                    .compile("([-+]?\\d*\\.?\\d+)?(<=|>=|<|>)?x(<=|>=|<|>)?([-+]?\\d*\\.?\\d+)?");
                Matcher matcherFormula = patternFormula.matcher(condition);

                if (matcherFormula.matches()) {
                    PiecewiseFormula item = new PiecewiseFormula();
                    item.operatorLeft = parseDouble(matcherFormula.group(1));
                    item.operatorRight = parseDouble(matcherFormula.group(4));
                    item.inequalityLeft = matcherFormula.group(2);
                    item.inequalityRight = matcherFormula.group(3);
                    item.result = parseDouble(resultStr);
                    retStruct.add(item);
                }
            }
        }
        return retStruct;
    }

    private static double parseDouble(String input) {
        try {
            return Double.parseDouble(input);
        }
        catch (Exception e) {
            return Double.NaN;
        }
    }

    private static class PiecewiseFormula {

        private double operatorLeft;

        private String inequalityLeft;

        private String inequalityRight;

        private double operatorRight;

        private double result;

    }

}
